@@ -862,23 +862,23 @@ class OMFITmhdin(OMFITnamelist):
         if 'magnetics' in update:
             R_flux_loop = self['IN3']['RSI']
             Z_flux_loop = self['IN3']['ZSI']
-            name_flux_loop = self['IN3']['LPNAME']
+            name_flux_loop = list(map(lambda x: x.strip(), self['IN3']['LPNAME']))
             R_magnetic = self['IN3']['XMP2']
             Z_magnetic = self['IN3']['YMP2']
             A_magnetic = self['IN3']['AMP2']
             S_magnetic = self['IN3']['SMP2']
-            name_magnetic = self['IN3']['MPNAM2']
+            name_magnetic = list(map(lambda x: x.strip(), self['IN3']['MPNAM2']))
             with omas_environment(ods, cocosio=1):
                 for k, (r, z, name) in enumerate(zip(R_flux_loop, Z_flux_loop, name_flux_loop)):
                     ods[f'magnetics.flux_loop.{k}.name'] = name
-                    ods[f'magnetics.flux_loop.{k}.identifier'] = 'F_' + name
+                    ods[f'magnetics.flux_loop.{k}.identifier'] = name
                     ods[f'magnetics.flux_loop.{k}.position[0].r'] = r
                     ods[f'magnetics.flux_loop.{k}.position[0].z'] = z
                     ods[f'magnetics.flux_loop.{k}.type.index'] = 1
 
                 for k, (r, z, a, s, name) in enumerate(zip(R_magnetic, Z_magnetic, A_magnetic, S_magnetic, name_magnetic)):
                     ods[f'magnetics.b_field_pol_probe.{k}.name'] = name
-                    ods[f'magnetics.b_field_pol_probe.{k}.identifier'] = 'B_' + name
+                    ods[f'magnetics.b_field_pol_probe.{k}.identifier'] = name
                     ods[f'magnetics.b_field_pol_probe.{k}.position.r'] = r
                     ods[f'magnetics.b_field_pol_probe.{k}.position.z'] = z
                     ods[f'magnetics.b_field_pol_probe.{k}.length'] = s
@@ -934,9 +934,11 @@ class OMFITnstxMHD(SortedDict, OMFITascii):
 
     @dynaLoad
     def load(self):
+        self.clear()
         with open(self.filename, 'r') as f:
             lines = f.read().split('\n')
         lines = [line for line in lines if len(line.strip()) and not line.startswith(';')]
+        self['all'] = {}
         for line in lines:
             line, description = (line + ';').split(';', 1)
             line = line.split()
@@ -968,14 +970,30 @@ class OMFITnstxMHD(SortedDict, OMFITascii):
                     self[line[1]][piece][item] = ast.literal_eval(self[line[1]][piece][item])
                 except (ValueError, SyntaxError):
                     pass
+            self['all'][line[0]] = self[line[1]][piece]
 
         # postprocess
         if self.type == 'signals':
             mds_tree_map = {'o': 'operations', 'e': 'engineering', 'a': 'activespec', 'r': 'radiatiion', 'E2': 'efit02', '-': 'computed'}
+            for group in self:
+                if group not in ['all']:
+                    for item in self[group]:
+                        self[group][item]['mds_tree'] = mds_tree_map[self[group][item]['mds_tree']]
+            # sort entries according to their `map` fields
+            self['mappings'] = {}
             for group in self:
                 for item in self[group]:
-                    self[group][item]['mds_tree'] = mds_tree_map[self[group][item]['mds_tree']]
-
+                    map_var = self[group][item].get('map_var', 'none')
+                    self['mappings'].setdefault(map_var, {})
+                    map_index = self[group][item].get('map_index', len(self['mappings'][map_var]))
+                    self['mappings'][map_var][map_index] = value = self[group][item]
+                    if 'mds_tree' in value:
+                        if value['mds_tree'] == 'computed' and value['mds_name'] in self['all']:
+                            value['mds_tree_resolved'] = self['all'][value['mds_name']]['mds_tree']
+                            value['mds_name_resolved'] = self['all'][value['mds_name']]['mds_name'] + '/' + str(value['scale'])
+                        else:
+                            value['mds_tree_resolved'] = value['mds_tree']
+                            value['mds_name_resolved'] = value['mds_name']
         return self
 
     def pretty_print(self):
