@@ -238,7 +238,7 @@ def tunneled_MDSserver(MDSserver, quiet=True, forceRemote=False):
 
 
 def MDSplusConnection(server, cached=True, quiet=True):
-    '''
+    """
     Return a low-level MDSplus.Connection object to the specified MDS+ server
     taking care of establising a tunneled connection if necessary
 
@@ -249,7 +249,7 @@ def MDSplusConnection(server, cached=True, quiet=True):
     :param quiet: verbose or not
 
     :return: MDSplus.Connection object
-    '''
+    """
     server0 = translate_MDSserver(server, '')
     tunneled_server = tunneled_MDSserver(server0, quiet=quiet)
     if cached:
@@ -267,15 +267,15 @@ class OMFITmdsObjects(object):
         # these are the server, treename and shot exactly as input by the user
         if server in ['None', 'none']:
             server = None
-        self.server = server
+        self.server = evalExpr(server)
         if treename in ['None', 'none']:
             treename = None
-        self.treename = treename
+        self.treename = evalExpr(treename)
         if shot in ['None', 'none']:
             shot = None
         if isinstance(shot, str):
             shot = int(shot)
-        self.shot = shot
+        self.shot = evalExpr(shot)
 
         # self server0, treename0 and shot0 that will be used internally by OMFIT
         # can fail if called when loading a saved mds object prior to the MainSettings server expressions
@@ -354,8 +354,8 @@ class OMFITmds(SortedDict, OMFITmdsObjects):
         SortedDict.__init__(self, caseInsensitive=True)
         OMFITmdsObjects.__init__(self, server, treename, shot)
         self.dynaLoad = True
-        self.subtree = subtree
-        self.caching = caching
+        self.subtree = evalExpr(subtree)
+        self.caching = evalExpr(caching)
         self.quiet = quiet
 
     @dynaLoad
@@ -652,7 +652,7 @@ class OMFITmdsConnectionBaseClass(SortedDict):
                     raise
 
     def write(self, server, treename, shot, node, data, error=None, units=None):
-        '''
+        """
         Write the data to MDS+
 
         :param server: The MDS+ server
@@ -671,7 +671,7 @@ class OMFITmdsConnectionBaseClass(SortedDict):
         :param units: units to be written (ignored if data is DataArray)
 
         :param quantities: list of quantities to write
-        '''
+        """
         mds_connection_cache_ID = ('write', server, treename, shot)
         if mds_connection_cache_ID not in self:
             self[mds_connection_cache_ID] = MDSplus.Connection(server)
@@ -709,7 +709,7 @@ class OMFITmdsConnectionBaseClass(SortedDict):
             raise _excp.__class__(f'\n{node} {txt}\n' + str(_excp))
 
     def write_dataset(self, server, treename, shot, subtree, xarray_dataset, quantities=None, translator=None):
-        '''
+        """
         Write the contents of xarray_dataset to MDS+, with smart referencing of dimensions
 
         :param server: The MDS+ server
@@ -725,7 +725,7 @@ class OMFITmdsConnectionBaseClass(SortedDict):
         :param quantities: list of quantities to write
 
         :param translator: a function that translates nodes from their dataset value to a string shorter than 12 chars
-        '''
+        """
         if translator is None:
             translator = lambda x: x
 
@@ -788,7 +788,7 @@ class OMFITmdsConnectionBaseClass(SortedDict):
         return dim_quantities + var_quantities
 
     def create_model_tree(self, server, treename, subtree, data, clear_subtree=False):
-        '''
+        """
         Create new nodes OMFIT.subtree.<data.keys()> on server
 
         :param server: The MDS+ server (OMFIT serverish)
@@ -798,7 +798,7 @@ class OMFITmdsConnectionBaseClass(SortedDict):
         :param subtree: The string describing the code or workflow, e.g. "ONETWO" (less than twelve characters)
 
         :param clear_subtree: Whether to clear the subtree node
-        '''
+        """
         if len(treename) > 12:
             raise ValueError('MDS+ treename can only be 12 characters long: ' + treename)
         if len(subtree) > 12:
@@ -870,9 +870,9 @@ _OMFITmdsConnectionBaseObjects = {}
 
 
 def OMFITmdsConnectionBase(*args, **kw):
-    '''
+    """
     returns OMFITmdsConnectionBaseClass objects, in a way that is safe for each parallel process
-    '''
+    """
     tmp = '_'.join(map(str, OMFITaux.setdefault('prun_process', [])))
     if tmp not in _OMFITmdsConnectionBaseObjects:
         _OMFITmdsConnectionBaseObjects[tmp] = OMFITmdsConnectionBaseClass(*args, **kw)
@@ -881,7 +881,7 @@ def OMFITmdsConnectionBase(*args, **kw):
 
 class OMFITcaching(SortedDict):
     def __init__(self, type, cachesDir=True, limit=1000):
-        '''
+        """
         :param type: string with cache type
 
         :param cachesDir: directory to use for off-line storage of data
@@ -890,7 +890,7 @@ class OMFITcaching(SortedDict):
                           If a string, then that value is used
 
         :param limit: limit number of elements for in-memory caching
-        '''
+        """
         SortedDict.__init__(self, limit=limit)
         self._cachesDir = cachesDir
         self.type = type
@@ -1060,9 +1060,9 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
 
     # DATA (can be overridden by subclassing --> _base_data)
     def data(self):
-        '''
+        """
         :return: array with the data
-        '''
+        """
         return self._base_data()
 
     def _base_data(self):
@@ -1106,9 +1106,9 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
 
     # ERROR (can be overridden by subclassing --> _base_error)
     def error(self):
-        '''
+        """
         :return: array with the error
-        '''
+        """
         return self._base_error()
 
     def _base_error(self):
@@ -1131,9 +1131,9 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
 
     # XARRAY DATA
     def xarray(self):
-        '''
+        """
         :return: DataArray with information from this node
-        '''
+        """
 
         if not self.check():
             raise ValueError('No data in %s on %s' % (self.TDI, self.idConnStr()))
@@ -1360,13 +1360,13 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
 
     # ---------
     def write(self, xarray_data):
-        '''
+        """
         write data to node
 
         :param xarray_data: multidimensional data to be written to MDS+ node in the format of an xarray.DataArray
 
         :return: outcome of MDSplus.put command
-        '''
+        """
 
         # handle ssh connection and expressions
         if not hasattr(self, 'server0'):
@@ -1423,7 +1423,7 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
                 return interpolate.interp1d(self.dim_of(1), data.T, kind=interp, bounds_error=bounds_error)(cut_value)
 
     def data(self, cut_value=None, cut_dim=0, interp='nearest', bounds_error=True):
-        '''
+        """
         :param cut_value: value of dimension `cut_dim` along which to perform the cut
 
         :param cut_dim: dimension along which to perform the cut
@@ -1434,7 +1434,7 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
 
         :return: If `cut_value` is None then return the MDS data.
                  If `cut_value` is not None, then return data value at `cut_value` for dimension `cut_dim`
-        '''
+        """
         data = self._base_data()
 
         if cut_value is None or data is None:
@@ -1443,7 +1443,7 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
             return self._slice(data, cut_value=cut_value, cut_dim=cut_dim, interp=interp, bounds_error=bounds_error)
 
     def error(self, cut_value=None, cut_dim=0, interp='nearest', bounds_error=True):
-        '''
+        """
         :param cut_value: value of dimension `cut_dim` along which to perform the cut
 
         :param cut_dim: dimension along which to perform the cut
@@ -1454,7 +1454,7 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
 
         :return: If `cut_value` is None then return the MDS error.
                  If `cut_value` is not None, then return error value at `cut_value` for dimension `cut_dim`
-        '''
+        """
         error = self._base_error()
 
         if cut_value is None or error is None:
@@ -1552,11 +1552,11 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
             return not (bad_mdsdata or bad_ptdata or bad_mdsdim or bad_ptdim)
 
     def plot(self, *args, **kw):
-        '''
+        """
         plot method based on xarray
 
         :return: current axes
-        '''
+        """
         from matplotlib import pyplot
 
         tmp = self.xarray()
