@@ -13,14 +13,21 @@ except ImportError as _excp:
         raise
 
 from omfit_classes.utils_math import *
+from omfit_classes.omfit_base import OMFITtree
+from omfit_classes.omfit_osborne import OMFITpFile
+from omfit_classes.omfit_eqdsk import OMFITgeqdsk
 from omfit_classes.omfit_data import OMFITncDataset, OMFITncDynamicDataset, importDataset
 from omas import ODS, omas_environment, cocos_transform, define_cocos
 from omfit_classes.omfit_omas_utils import add_generic_OMFIT_info_to_ods
 
 import inspect
 import numpy as np
+from uncertainties import unumpy, ufloat
+from uncertainties.unumpy import std_devs, nominal_values
 from scipy import constants
 
+np.seterr(invalid='ignore', divide='ignore')
+
 __all__ = ['OMFITprofiles', 'OMFITprofilesDynamic']
 
 model_tree_species = ['e', '2H1', '4He2', '6Li3', '10B5', '12C6', '14N7', '20Ne10']
@@ -34,12 +41,75 @@ model_tree_quantities = ['angular_momentum', 'angular_momentum_density', 'angula
 'Total_Zeff', 'V_pol_{species}_KDG', 'Zavg_Ni', 'Zavg_W', 'Zeff']
 # fmt: on
 
-class OMFITprofiles(OMFITncDataset):
-    def __init__(self, filename, data_vars=None, coords=None, attrs=None, comment=''):
-        '''
-        Data class used by OMFITprofiles, CAKE and other
-        OMFIT modules for storing experimental profiles data
 
+def ugrad1(a2d):
+    """
+    Gradient along second axis with uncertainty propagation.
+    :param a2d: 2D array or uarray
+    :return:
+    """
+    if isinstance(a2d, DataArray):
+        a2d = a2d.values
+    if is_uncertain(a2d):
+        dy = np.gradient(nominal_values(a2d), axis=1)
+        ye = std_devs(a2d)
+        sigma = np.zeros_like(ye)
+        sigma[:, 1:-1] = 0.5 * np.sqrt(ye[:, :-2] ** 2 + ye[:, 2:] ** 2)
+        sigma[:, 0] = 0.5 * np.sqrt(ye[:, 0] ** 2 + ye[:, 1] ** 2)
+        sigma[:, -1] = 0.5 * np.sqrt(ye[:, -2] ** 2 + ye[:, -1] ** 2)
+        result = unumpy.uarray(dy, sigma)
+    else:
+        result = np.gradient(a2d, axis=1)
+    return result
+
+
+def mZ(species):
+    """
+    Parse subscript strings and return ion mass and charge
+
+    :param species: subscript strings such as `e`, `12C6`, `2H1`, 'fast_2H1`, ...
+
+    :return: m and Z
+    """
+    species = str(species).replace('fast_', '')
+    if species == 'e':
+        Z = -1
+        m = constants.m_e
+    else:
+        m = int(re.sub('([0-9]+)([a-zA-Z]+)([0-9]+)', r'\1', species))
+        name = re.sub('([0-9]+)([a-zA-Z]+)([0-9]+)', r'\2', species)
+        Z = int(re.sub('([0-9]+)([a-zA-Z]+)([0-9]+)', r'\3', species))
+        m *= constants.m_u
+    return m, Z
+
+
+def get_species(derived):
+    """
+    Identify species and ions that have density information
+    """
+    species = []
+    for key in list(derived.data_vars.keys()):
+        if not re.match('^[nT](_fast)?_([0-9]+[a-zA-Z]+[0-9]|e)$', key):
+            continue
+        s = key.split('_')[-1]
+        if '_fast_' in key:
+            s = 'fast_' + s
+        species.append(s)
+    species = tolist(np.unique(species))
+    ions = [s for s in species if s not in ['e']]
+    ions_with_dens = [i for i in ions if 'n_' + i in derived]
+    ions_with_fast = [i.replace('fast_', '') for i in ions if 'fast_' in i]
+    return species, ions, ions_with_dens, ions_with_fast
+
+
+class OMFITprofiles(OMFITncDataset):
+    """
+    Data class used by OMFITprofiles, CAKE and other
+    OMFIT modules for storing experimental profiles data
+    """
+
+    def __init__(self, filename, data_vars=None, coords=None, attrs=None, comment=''):
+        """
         :param filename: filename of the NetCDF file where data will be saved
 
         :param data_vars: see xarray.Dataset
@@ -49,7 +119,7 @@ class OMFITprofiles(OMFITncDataset):
         :param attrs: see xarray.Dataset
 
         :param comment: String that if set will show in the OMFIT tree GUI
-        '''
+        """
         self.dynaLoad = False
         super().__init__(filename, data_vars=data_vars, coords=coords, attrs=attrs)
         self.OMFITproperties['comment'] = comment
@@ -68,50 +138,19 @@ class OMFITprofiles(OMFITncDataset):
         else:
             return super().__tree_repr__()
 
-    def mZ(self, species):
-        '''
-        Parse subscript strings and return ion mass and charge
-
-        :param species: subscript strings such as `e`, `12C6`, `2H1`, 'fast_2H1`, ...
-
-        :return: m and Z
-        '''
-        species = str(species)
-        if species == 'e':
-            Z = -1
-            m = constants.m_e
-        else:
-            m = int(re.sub('([0-9]+)([a-zA-Z]+)([0-9]+)', r'\1', species))
-            name = re.sub('([0-9]+)([a-zA-Z]+)([0-9]+)', r'\2', species)
-            Z = int(re.sub('([0-9]+)([a-zA-Z]+)([0-9]+)', r'\3', species))
-            m *= constants.m_u
-        return m, Z
-
-    def get_species(self):
-        """
-        Identify species and ions that have density information
-        """
-        species = []
-        for key in list(self.data_vars.keys()):
-            if key.count('_') != 1:
-                continue
-            s = re.sub('^[nT]_(.*)', r'\1', key)
-            if s in ['e', 'b'] or re.match('[0-9]+[a-zA-Z]+[0-9]+', s):
-                species.append(s)
-        species = tolist(np.unique(species))
-        ions = [s for s in species if s not in ['e']]
-        ions_with_dens = [i for i in ions if 'n_' + i in self]
-        return species, ions, ions_with_dens
-
     def to_omas(self, ods=None, times=None):
-        '''
+        """
         :param ods: ODS to which data will be appended
 
         :return: ods
-        '''
+        """
 
         if ods is None:
             ods = ODS()
+            eq_times = None
+        else:
+            # Determine if equilibrium is avaliable and for what times.
+            eq_times = ods.time('equilibrium') * 1e3  # ODS times are in s, and omfit_profiles are in ms.
 
         if 'device' in self.attrs:
             ods['dataset_description.data_entry.machine'] = self.attrs['device']
@@ -126,16 +165,32 @@ class OMFITprofiles(OMFITncDataset):
             raise ValueError("Fit coordinate should be 'rho' or 'psi_n'")
 
         # general info
-        species, ions, ions_with_dens = self.get_species()
+        species, ions, ions_with_dens, ions_with_fast = get_species(self)
         nion = len(ions)
-        times = self['time'].values
+        if times is None:
+            times = self['time'].values
+        else:
+            times = np.atleast_1d(times)
+
+        # figure out time match between eq (if available) and profiles
+        if eq_times is None:
+            printw("No equilibrium data is avaliable to to_omas(). Some info will not be stored.")
+        elif np.all([time in eq_times for time in times]):  # aka is all times have avaliable eq
+            printd("Matching equilibrium times found")
+        elif np.any([time in eq_times for time in times]):
+            printw("Some time slices don't have corresponding equilibria!")
+            printw("These time slices will be missing some information.")
+        else:
+            printw("No equilibrium data is avaliable to to_omas(). Some info will not be stored.")
 
         # assign both core_profies and edge_profiles but with data from different spatial ranges
         for coredgestring in ['core_profiles', 'edge_profiles']:
             if coredgestring.startswith('core'):
                 derived = self.where(self[fit_coordinate] <= 1.0, drop=True)
+                core = True
             else:
                 derived = self.where(self[fit_coordinate] > 0.8, drop=True)
+                core = False
 
             coredge = ods[coredgestring]
             prop = coredge['ids_properties']
@@ -147,6 +202,27 @@ class OMFITprofiles(OMFITncDataset):
             for ti, time in enumerate(times):
                 profs = coredge[f'profiles_1d.{ti}']
                 profs['time'] = time / 1e3
+
+                # get corresponding eq and extract needed info
+                geq = None
+                R = None
+                Bt = None
+                Bp = None  # Make sure we don't accidentally use values from last timeslice
+                if eq_times is not None and len(eq_times) > 0 and time in eq_times:
+                    i_eq = np.where(time == eq_times)[0][0]  # just the first index
+                    geq = OMFITgeqdsk('g0.0').from_omas(ods, time_index=i_eq)
+                    # Not the most efficient but can access exiting functions better.
+                    if fit_coordinate == 'psi_n':
+                        psin = derived['psi_n'].values
+                    else:
+                        psin = derived['psi_n'].sel(time=time).values
+                    psin_eq = np.linspace(0.0, 1.0, len(geq['PRES']))
+
+                    # Re-interpolate
+                    R = np.interp(psin, psin_eq, geq['fluxSurfaces']['midplane']['R'])
+                    Bt = np.interp(psin, psin_eq, geq['fluxSurfaces']['midplane']['Bt'])
+                    Bp = np.interp(psin, psin_eq, geq['fluxSurfaces']['midplane']['Bp'])
+
                 for q in derived.variables:
                     fit = derived[q]
                     if q in ['time']:
@@ -165,12 +241,15 @@ class OMFITprofiles(OMFITncDataset):
                         profs['electrons.density_thermal'] = fit.sel(time=time).values
                     elif q == 'T_e':
                         profs['electrons.temperature'] = fit.sel(time=time).values
+                    elif q == 'omega_P_e' and core:  # this location is not valid OMAS location for edge profiles
+                        profs['electrons.rotation.diamagnetic'] = fit.sel(time=time).values
                     elif '_' in q and q.split('_', 1)[1] in ions:
                         continue
 
                 # thermal ions
-                for ni, ion in enumerate(ions[::-1]):
-                    if ion == 'b':
+                ni = 0
+                for ion in ions[::-1]:
+                    if ion == 'b' or ion.startswith('fast_'):
                         continue
                     profi = profs[f'ion.{ni}']
                     profi['density_thermal'] = derived[f'n_{ion}'].sel(time=time).values
@@ -187,27 +266,55 @@ class OMFITprofiles(OMFITncDataset):
                     if f'V_tor_{ion}' in derived and not (f'omega_tor_{ion}' in derived and 'R_midplane' in derived):
                         profi['velocity.toroidal'] = derived[f'V_tor_{ion}'].sel(time=time).values
                     elif f'omega_tor_{ion}' in derived and 'R_midplane' in derived:
-                        profi['velocity.toroidal'] = (derived[f'omega_tor_{ion}'].sel(time=time) * derived['R_midplane'].sel(time=time)).values
+                        profi['velocity.toroidal'] = (
+                            derived[f'omega_tor_{ion}'].sel(time=time) * derived['R_midplane'].sel(time=time)
+                        ).values
                     if f'V_pol_{ion}' in derived:
                         profi['velocity.poloidal'] = derived[f'V_pol_{ion}'].sel(time=time).values
+                    if core:  # extra rotation info for the core profiles. (Not valid nodes for edge)
+                        if f'omega_P_{ion}' in derived:
+                            profi['rotation.diamagnetic'] = derived[f'omega_P_{ion}'].sel(time=time).values
+                        if f'omega_tor_{ion}' in derived:
+                            profi['rotation_frequency_tor'] = derived[f'omega_tor_{ion}'].sel(time=time).values
+                        if f'V_pol_{ion}' in derived and Bp is not None:
+                            # Save to parallel streaming function, this will allow omegp to be calculated from ods
+                            profi['rotation.parallel_stream_function'] = derived[f'V_pol_{ion}'].sel(time=time).values / Bp
 
-                # beam ions
+                    # Advance ni; its important that if it is fast ion and the loop-iteration is skipped, then ni do not advance
+                    ni += 1
+
+                # fast ions
                 for ion in ions:
-                    if ion != 'b':
+                    if ion != 'b' and not ion.startswith('fast_'):
                         continue
+                    # Get the 'base' ion for the fast population
+                    if ion.startswith('fast_'):
+                        base_ion = ion.replace('fast_', '')
+                    elif ion == 'b':
+                        base_ion == '2H1'  # back compat for '_b' notations
+                    ion_details = list(atomic_element(symbol=base_ion).values())[0]
+
+                    # Determin the corresponding ion index
                     ni = len(profs['ion'])
                     for nii in profs['ion']:
-                        profi = profs[f'ion.{ni}']
-                        if profi['label'] == 'H':
-                            ion = '2H1'
+                        profi = profs[f'ion.{nii}']
+                        if profi['label'] == ion_details['symbol']:
                             ni = nii
                             break
                     profi = profs[f'ion.{ni}']
+
+                    # Add fast_ion data.
                     profi['density_fast'] = derived[f'n_{ion}'].sel(time=time).values
-                    pfast = derived[f'T_{ion}'].sel(time=time).values * constants.e * derived[f'n_{ion}'].sel(time=time).values
-                    profi['pressure_fast_perpendicular'] = 2.0 / 3.0 * pfast
+                    if f'p_{ion}' in derived:
+                        pfast = derived[f'p_{ion}'].sel(time=time)  #'ion' here would have the form 'fast_2H1' for example
+                    else:
+                        pfast = derived[f'T_{ion}'].sel(time=time).values * constants.e * derived[f'n_{ion}'].sel(time=time).values
+                    profi['pressure_fast_perpendicular'] = (
+                        1.0 / 3.0 * pfast
+                    )  # Assume isotropic fast ions. Also OMAS treats p_xxx_perp as pressure in one of the perp directions, I think.
                     profi['pressure_fast_parallel'] = 1.0 / 3.0 * pfast
-                    ion_details = list(atomic_element(symbol=ion).values())[0]
+
+                    # Attach atomic data (from base_ion)
                     profi['label'] = ion_details['symbol']
                     profi['z_ion'] = float(ion_details['Z_ion'])
                     profi['multiple_states_flag'] = 0
@@ -218,16 +325,20 @@ class OMFITprofiles(OMFITncDataset):
 
                 if 'Total_Zeff' in derived:
                     profs['zeff'] = derived['Total_Zeff'].sel(time=time).values
+
+        # Populate total pressure nodes under 'profiles_1d`
+        ods.physics_core_profiles_pressures()
+        # ods.physics_edge_profiles_pressures() # This function does not exist, but really should.
         return ods
 
     def model_tree_quantities(self):
-        '''
+        """
         Returns list of MDS+ model_tree_quantities for all species.
         The function will warn if some of the `model_tree_quantities` are missing in OMFIT-source/omfit/omfit_classes/omfit_profiles.py
         and the model tree should be updated
 
         :return: list of strings
-        '''
+        """
         new_model_tree_quantities = set(model_tree_quantities)
         for item in self.variables:
             match = False
@@ -240,16 +351,17 @@ class OMFITprofiles(OMFITncDataset):
                 new_model_tree_quantities.add(tmp)
             else:
                 new_model_tree_quantities.add(item)
-        new_model_tree_quantities = sorted(list(new_model_tree_quantities),key=lambda x:x.lower())
+        new_model_tree_quantities = sorted(list(new_model_tree_quantities), key=lambda x: x.lower())
         if new_model_tree_quantities != model_tree_quantities:
             import textwrap
+
             printe('WARNING!: Update model_tree_quantities in OMFIT-source/omfit/omfit_classes/omfit_profiles.py')
             printe('WARNING!: and update the OMFIT_PROFS MDS+ model tree')
-            printe('-'*140)
+            printe('-' * 140)
             printe('# fmt: off')
             printe(textwrap.fill(f'model_tree_quantities = {repr(new_model_tree_quantities)}', width=140))
             printe('# fmt: on')
-            printe('-'*140)
+            printe('-' * 140)
 
         quantities = []
         for item in new_model_tree_quantities:
@@ -261,14 +373,15 @@ class OMFITprofiles(OMFITncDataset):
         return quantities
 
     def create_model_tree(self, server, treename='OMFIT_PROFS'):
-        '''
+        """
         Generate MDS+ model tree
 
         :param server: MDS+ server
 
         :param treename: MDS+ treename
-        '''
+        """
         from omfit_classes.omfit_mds import OMFITmdsConnection
+
         conn = OMFITmdsConnection(server)
 
         quantities = {self.mds_translator(k): None for k in self.model_tree_quantities()}
@@ -276,7 +389,7 @@ class OMFITprofiles(OMFITncDataset):
         conn.create_model_tree(treename, '', quantities, clear_subtree=True)
 
     def to_mds(self, server, shot, treename='OMFIT_PROFS'):
-        '''
+        """
         This script writes the OMFITproflies datase to DIII-D MDS+ and updates d3drdb accordingly
 
         :param server: MDS+ server
@@ -286,14 +399,16 @@ class OMFITprofiles(OMFITncDataset):
         :param treename: MDS+ treename
 
         :return: runid, treename
-        '''
+        """
         from omfit_classes.omfit_mds import OMFITmdsConnection
+
         conn = OMFITmdsConnection(server)
 
         quantities = self.model_tree_quantities()
 
         # find next available runid in d3drdb for this shot
         from omfit_classes.omfit_rdb import OMFITrdb
+
         rdb = OMFITrdb(db='code_rundb', server='d3drdb', by_column=True)
         runs = rdb.select(f"select run_id from plasmas where shot={shot} and code_name='OMFITprofiles'", by_column=True)
         for run in range(1, 9999):
@@ -303,33 +418,42 @@ class OMFITprofiles(OMFITncDataset):
         print(f'Writing OMFITprofiles to MDS+ {runid}')
 
         # write to MDS+
-        quantities = conn.write_dataset(treename=treename, shot=runid, subtree='', xarray_dataset=self, quantities=quantities, translator=lambda x:self.mds_translator(x))
+        quantities = conn.write_dataset(
+            treename=treename,
+            shot=runid,
+            subtree='',
+            xarray_dataset=self,
+            quantities=quantities,
+            translator=lambda x: self.mds_translator(x),
+        )
         conn.write(treename=treename, shot=runid, node='__content__', data=';'.join(quantities))
 
         # add data to d3drdb (after MDS+ so that this is done only if MDS+ write is successful)
-        data = {'code_name': 'OMFITprofiles',
-                'shot': shot,
-                'experiment': 'DIII-D',
-                'run_type': 'user',
-                'tree': treename,
-                'start_time': np.min(self['time'].values),
-                'stop_time': np.max(self['time'].values),
-                'mdsserver': server,
-                'run_id': runid}
+        data = {
+            'code_name': 'OMFITprofiles',
+            'shot': shot,
+            'experiment': 'DIII-D',
+            'run_type': 'user',
+            'tree': treename,
+            'start_time': np.min(self['time'].values),
+            'stop_time': np.max(self['time'].values),
+            'mdsserver': server,
+            'run_id': runid,
+        }
         rdb.insert('plasmas', data, verbose=False)
         pprint(data)
 
         return runid, treename
 
     def mds_translator(self, inv=None):
-        '''
+        """
         Converts strings OMFITprofiles dataset keys to MDS+ nodes less than 12 chars long
 
         :param inv: tring to which to apply the transformation
                     if `None` the transformation is applied to all of the OMFITprofiles.model_tree_quantities for sanity check
 
         :return: transformed sting or if inv is None the `mapped_model_2_mds` and `mapped_mds_2_model` dictionaries
-        '''
+        """
         mapper = SortedDict()
         mapper['angular_momentum_density'] = 'mom_dens'
         mapper['angular_momentum'] = 'mom'
@@ -350,7 +474,9 @@ class OMFITprofiles(OMFITncDataset):
             for match, sub in mapper.items():
                 inv = inv.replace(match, sub)
             if len(inv) > 12:
-                raise Exception(f'MDS+ OMFITprofiles quantity is longer than 12 chars: {inv}\nUpdate the mds_translator function accordingly')
+                raise Exception(
+                    f'MDS+ OMFITprofiles quantity is longer than 12 chars: {inv}\nUpdate the mds_translator function accordingly'
+                )
             return inv
         else:
             model_tree_quantities = self.model_tree_quantities()
@@ -370,6 +496,194 @@ class OMFITprofiles(OMFITncDataset):
                 mapped_mds_2_model[item] = item0
             return mapped_model_2_mds, mapped_mds_2_model
 
+    def to_pFiles(self, eq, times=None, shot=None):
+        """
+        :param eq: ODS() or dict. (OMFITtree() is a dict)  Needs to contain equilibria information, either in the form
+            of the ODS with needed eq already loaded, or as OMFITgeqdsk() objects in the Dict with the time[ms] as
+            keys. Times for the eq need to be strict matches to profiles times coord.
+
+        :param times: array like. time for which you would like p files to be generated.
+
+        :param shot: int. shot number, only relevant in generating p file names.
+
+        :return: OMFITtree() containing a series of OMFITpfile objs.
+        """
+
+        # Generate times if needed
+        if times is None:
+            try:
+                times = self['time'].values
+            except KeyError:
+                # Just here to add a helpful hint
+                printw("Looking like your OMFITprofiles obj is missing 'time'. Is it properly initialized?")
+                raise
+
+        # Get shot if it can be found
+        if shot is None:
+            if 'shot' in self.attrs:
+                shot = self.attrs['shot']
+            else:
+                shot = 0  # Dummy number for p file names.
+
+        # get good_times
+        good_times = []
+        if isinstance(eq, ODS):
+            # Check times
+            good_times = [t for t in times if t in eq.time('equilibrium')]  # ODS().time('...') will throw error if time is inconsistent
+
+        elif isinstance(eq, dict):
+            for i, time in enumerate(times):
+                d3_time = int(time)
+                if d3_time in eq:
+                    good_times.append(time)
+                else:
+                    printw(f"Missing eq data for {time}, it will be skipped!")
+        else:
+            printw("Input arg 'eq' is in unrecognized format. This will fail!")
+
+        good_times = [t for t in good_times if t in self['time'].values]
+        good_times = array(good_times)
+
+        if len(good_times) == 0:
+            printw("No valid time found! Each timesilce needs profiles and equilibrium.")
+            return
+        else:
+            printd(f"The following time was found to be good: {good_times}")
+            printd("pFiles will be produced for these times only.")
+
+        ods = ODS()
+        # Now inject eq info, but only for good_times
+        if isinstance(eq, ODS):
+            prof_ods.equilibrium.time = np.array(good_times)
+            for i, time in enumerate(good_times):
+                j = where(eq.times('equilibrium') == time)
+                ods[f'equilibrium.time_slice.{i}'] = eq[f'equilibrium.time_slice.{j}']
+
+            prof_ods.physics_consistent_times()  # Generate missing time array
+        elif isinstance(eq, dict):
+            for i, time in enumerate(good_times):
+                d3_time = int(time)
+                ods = eq[d3_time].to_omas(ods=ods, time_index=i)
+                # This is not very efficient, but does make to_omas() more standardized. Open to change as needs change.
+        ods = self.to_omas(times=good_times, ods=ods)
+
+        out_tree = OMFITtree()
+        for i, time in enumerate(good_times):
+            d3_time = int(time)
+            out_tree[d3_time] = OMFITpFile(f'p{shot:06}.{d3_time:05}').from_omas(ods, time_index=i)
+
+        return out_tree
+
+    def get_xkey(self):
+        """
+        Get the key of the x-coord associated with this data array.
+
+        :returns: str. key of the x-coord associated with the profiles, like 'psi_n'.
+        """
+        dims = list(self.dims.keys())
+        return dims[dims.index('time') - 1]
+
+    def diamagnetic_frequencies(self, spc, update=True):
+        """
+        Calculate the diamagnetic frequency, and its density / temperature components.
+
+        Formula: \omega_P = -\frac{T}{nZe}\frac{dn}{d\psi} - \frac{1}{Ze}\frac{dT}{d\psi}
+
+        :param spc: Species for which temperature is fit
+
+        :param update: bool. Set to true to update self, if False, only returns and does not update self.
+            Gradients, if missing, will always update though.
+
+        """
+        freqs = xarray.Dataset()
+
+        m, Z = mZ(spc)
+
+        # calculate gradients if not avaliable
+        if f'dn_{spc}_dpsi' not in self and f'n_{spc}' in self:
+            dn_dpsi = self.xderiv('n_' + spc, coord='psi')
+
+        if f'dT_{spc}_dpsi' not in self and f'T_{spc}' in self:
+            dT_dpsi = self.xderiv('T_' + spc, coord='psi')
+
+        # density part
+        if self.check_keys(keys=[f'n_{spc}', f'T_{spc}'] + [f'dn_{spc}_dpsi'], name=f'omega_N_{spc}'):
+            omega_N = -self[f'dn_{spc}_dpsi'] * self[f'T_{spc}'] * constants.eV / (self[f'n_{spc}'] * Z * constants.e)
+            omega_N.attrs['long_name'] = r"$\omega_N = -\frac{T}{nZe}\frac{dn}{d\psi}$"
+            freqs.update({f'omega_N_{spc}': omega_N})
+        # temperature part
+        if self.check_keys(keys=[f'T_{spc}'] + [f'dT_{spc}_dpsi'], name=f'omega_T_{spc}'):
+            omega_T = -self[f'dT_{spc}_dpsi'] * constants.eV / (Z * constants.e)
+            omega_T.attrs['long_name'] = r"$\omega_T = -\frac{1}{Ze}\frac{dT}{d\psi}$"
+            freqs.update({f'omega_T_{spc}': omega_T})
+        # total
+        if len(freqs.data_vars) == 2:
+            omega_P = omega_N + omega_T
+            omega_P.attrs['long_name'] = r"$\omega_{p," + spc + r"} = -\frac{T}{nZe}\frac{dn}{d\psi} -\frac{1}{Ze}\frac{dT}{d\psi}$"
+            freqs.update({f'omega_P_{spc}': omega_P})
+
+        if update:
+            self.update(freqs)
+        return freqs
+
+    def xderiv(self, key, coord='psi', update=True):
+        """
+        Returns the derivative of the value corresponding to key on the spatial coordinate coord.
+
+        :param key: str. The variable
+
+        :param coord: str. The radial coordinate with respect to which the derivative is taken
+
+        :param update: bool. Set to true to update self, if False, only returns and does not update self.
+
+        :return: Dataset
+        """
+        result = xarray.Dataset()
+        xkey = self.get_xkey()
+        if not self.check_keys([key, coord], name=f'd{key}_d{coord}'):
+            return result
+        dc = np.gradient(np.atleast_2d(self[coord].values), axis=-1)
+        dkey = 'd' + key + '_d' + xkey
+        if dkey in self and any(isfinite(v) for v in nominal_values(self[dkey].values).flat) and coord != xkey:
+            # if we have derivative UQ from the fit method, keep it and just swap coordinates
+            dx = np.gradient(np.atleast_2d(self[xkey].values), axis=-1)
+            dkdc = DataArray(self[dkey] * (dx / dc), coords=self[key].coords)
+        else:
+            # otherwise we have to calculate the derivative numerically - huge uncertainties when propagated
+            try:
+                dkdc = DataArray(ugrad1(self[key].values) / dc, coords=self[key].coords)
+            except ZeroDivisionError:  # This happens if dc array have 0s and self[key].values is a uarray/has uncertainty.
+                i_bad = dc == 0
+                dc[i_bad] = 1
+                dkdc_arr = ugrad1(self[key].values) / dc
+                dkdc_arr[i_bad] = ufloat(np.nan, np.nan)
+                dkdc = DataArray(ugrad1(self[key].values) / dc, coords=self[key].coords)
+        if coord in ['rho', 'psi_n']:
+            dkdc.attrs['units'] = self[key].attrs.get('units', '')
+        dkdc.attrs['long_name'] = r'd{:} / d$\{:}$'.format(key, coord)
+        result['d' + key + '_d' + coord] = dkdc
+
+        if update:
+            self.update(result)
+        return result
+
+    def check_keys(self, keys=[], name='', print_error=True):
+        """
+        Check to make sure required data is available
+        """
+        missing = []
+        for k in keys:
+            if not k in self:
+                missing.append(k)
+        if missing:
+            if not name:
+                name = 'value'
+            if print_error:
+                printw('  WARNING: Could not form {:}. Missing {:}'.format(name, ', '.join(['`%s`' % x for x in missing])))
+            return False
+        return True
+
+
 class OMFITprofilesDynamic(OMFITncDynamicDataset):
     """
     Class for dynamic calculation of derived quantities
@@ -391,7 +705,6 @@ class OMFITprofilesDynamic(OMFITncDynamicDataset):
     """
 
     def __init__(self, filename, fits=None, equilibrium=None, main_ion='2H1', **kw):
-
         OMFITncDataset.__init__(self, filename, **kw)
 
         if fits:
@@ -403,7 +716,7 @@ class OMFITprofilesDynamic(OMFITncDynamicDataset):
 
             # collect some meta data about which particle species have what info available
             self.attrs['main_ion'] = str(main_ion)
-            species, ions, ions_with_dens = self.get_species()
+            species, ions, ions_with_dens, ions_with_fast = get_species(self)
             self['species'] = DataArray(species, dims=['species'])
             ions += [self.attrs['main_ion']]
             self['ions'] = DataArray(ions, dims=['ions'])
@@ -475,41 +788,6 @@ class OMFITprofilesDynamic(OMFITncDynamicDataset):
         # return value
         return OMFITncDynamicDataset.__getitem__(self, key)
 
-    def mZ(self, species):
-        '''
-        Parse subscript strings and return ion mass and charge
-
-        :param species: subscript strings such as `e`, `12C6`, `2H1`, 'fast_2H1`, ...
-
-        :return: m and Z
-        '''
-        species = str(species).replace('fast_', '')
-        if species == 'e':
-            Z = -1
-            m = constants.m_e
-        else:
-            m = int(re.sub('([0-9]+)([a-zA-Z]+)([0-9]+)', r'\1', species))
-            name = re.sub('([0-9]+)([a-zA-Z]+)([0-9]+)', r'\2', species)
-            Z = int(re.sub('([0-9]+)([a-zA-Z]+)([0-9]+)', r'\3', species))
-            m *= constants.m_u
-        return m, Z
-
-    def get_species(self):
-        """
-        Identify species and ions that have density information
-        """
-        species = []
-        for key in list(self.data_vars.keys()):
-            if key.count('_') != 1:
-                continue
-            s = re.sub('^[nT]_(.*)', r'\1', key)
-            if s in ['e', 'b'] or re.match('[0-9]+[a-zA-Z]+[0-9]+', s):
-                species.append(s)
-        species = tolist(np.unique(species))
-        ions = [s for s in species if s not in ['e']]
-        ions_with_dens = [i for i in ions if 'n_' + i in self]
-        return species, ions, ions_with_dens
-
     def calc_n_main_ion(self):
         """
         Density of the main ion species.
@@ -519,11 +797,11 @@ class OMFITprofilesDynamic(OMFITncDynamicDataset):
 
         """
         main_ion = str(self.attrs['main_ion'])
-        mg, zg = self.mZ(main_ion)
+        mg, zg = mZ(main_ion)
         nz = self['n_e']
         for key in self['ions_with_dens'].values:
             if key != main_ion:
-                nz -= self['n_' + key].values * self.mZ(key)[1]
+                nz -= self['n_' + key].values * mZ(key)[1]
         self['n_' + main_ion] = nz / zg
         invalid = np.where(self['n_' + main_ion].values <= 0)[0]
         if len(invalid) > 0:
@@ -566,8 +844,8 @@ class OMFITprofilesDynamic(OMFITncDynamicDataset):
 
         """
         # calculate Zeff (not assuming quasi-neutrality)
-        nz_sum = np.sum([self['n_' + i].values * self.mZ(i)[1] for i in self['ions_with_dens'].values], axis=0)
-        nz2_sum = np.sum([self['n_' + i].values * self.mZ(i)[1] ** 2 for i in self['ions_with_dens'].values], axis=0)
+        nz_sum = np.sum([self['n_' + i].values * mZ(i)[1] for i in self['ions_with_dens'].values], axis=0)
+        nz2_sum = np.sum([self['n_' + i].values * mZ(i)[1] ** 2 for i in self['ions_with_dens'].values], axis=0)
         z_eff = nz2_sum / nz_sum + 0 * self['n_e'].rename('Zeff')
         z_eff.attrs['long_name'] = r'$Z_{eff}$'
         self['Zeff'] = z_eff
@@ -582,11 +860,11 @@ class OMFITprofilesDynamic(OMFITncDynamicDataset):
 
         """
         main_ion = str(self.attrs['main_ion'])
-        mg, zg = self.mZ(main_ion)
+        mg, zg = mZ(main_ion)
         nz = self['n_e']
         for key in self['ions_with_dens'].values:
             if key != main_ion:
-                nz -= self['n_' + key].values * self.mZ(key)[1]
+                nz -= self['n_' + key].values * mZ(key)[1]
         self['n_' + main_ion] = nz / zg
         invalid = np.where(self['n_' + main_ion].values <= 0)[0]
         if len(invalid) > 0:
