@@ -316,10 +316,8 @@ if os.path.exists(_userMainSettingsFilename):
             _tmp.load()
             recursiveUpdate(OMFIT['MainSettings'], _tmp)
 else:
+    # if MainSettingsNamelistDump.txt is not found then skeletonMainSettingsNamelist.txt is loaded
+    # this does not have customizations about usersname and such for connections to remote servers
+    # To create this file, start OMFIT frameowork and customize MainSettings via File >  Preferences GUI
     OMFIT['MainSettings'].update(NamelistFile(_skeleton_filename))
-    printe(
-        '\n  %s not found. \n'
-        '  Some of the OMFIT classes may not work properly until this file is created and customized. \n'
-        '  To create this file, start OMFIT frameowork and customize MainSettings via File >  Preferences GUI\n' % _userMainSettingsFilename
-    )
 SERVER = OMFIT['MainSettings']['SERVER']
